//--------------------------------------------------------------------------
// I N C L U D E S
#include <libps.h>
#include "pad.h"
#include "main.h" 

//--------------------------------------------------------------------------
// D E F I N E S 
#define TRUE 1
#define FALSE 0

#define DONE ((PadRead(1) & PADselect) && (PadRead(1) & PADstart))

//--------------------------------------------------------------------------
// G L O B A L S
int      output_buffer_index;         // buffer index
GsOT     world_ordering_table[2];     // ordering table headers
GsOT_TAG ordering_table[2][1<<1];     // actual ordering tables
PACKET   gpu_work_area[2][24000];     // GPU packet work area
volatile u_char *bb0, *bb1;           // joypad buffer pointers
u_char   prev_mode;                   // previous mode
int      fnt_id[9];                   // font id
int      debug = 0;                   // not used

//--------------------------------------------------------------------------
// P R O T O T Y P E (for my joypad routine)
int MyJoypad(int id);
static u_long PadRead(long id);

//--------------------------------------------------------------------------
// F U N C T I O N (for my joypad routine)
// note. joypad id is ignored in this demo(using joypad port 1)
// returns : direction(0 is center)
//    8 1 2
//    7 0 3
//    6 5 4
int MyJoypad(int id) {
  u_long padd = PadRead(1);

  if (padd & PADLup) {
    if (padd & PADLleft) return(8);                           
    if (padd & PADLright) return(2);                          
    return (1);                                                           
  } else if (padd & PADLright) {                              
    if (padd & PADLdown) return(4);                           
    return (3);                                                           
  } else if (padd & PADLdown) {                               
      if (padd & PADLleft) return(6);                         
      return (5);                                                         
  } else if (padd & PADLleft) return(7);                      
  return (0);                                                             
}                                                                         

//--------------------------------------------------------------------------
// M A I N
int main( void ) {
  int s;

  InitGame();

  printf("start\n");

  while( !DONE ) {
    s = MyJoypad(1);

    FntPrint(fnt_id[0], "~cfff Change-V's Joypad demo for PSX Yaroze");
    FntPrint(fnt_id[1], "~cfff   8 1 2\n\n   7 0 3\n\n   6 5 4");
    FntPrint(fnt_id[4], "~cfff direction : %d", s);
    FntPrint(fnt_id[5], "~cfff Push (Start) + (Select) to quit", s);

    UpdateScreen();
  }// end while loop

  DeInitGame();   // de-init the PlayStation
  return(0);      // success
}// end main 

//--------------------------------------------------------------------------
static u_long PadRead(long id) {
  return(~(*(bb0+3) | *(bb0+2) << 8 | *(bb1+3) << 16 | *(bb1+2) << 24));
}

//--------------------------------------------------------------------------
// Function: InitGame()
// Description: Initialize the graphics mode, joypad, ordering tables,
//              textures, and objects
// Parameters: none
// Returns: void
// Notes: N/A
//--------------------------------------------------------------------------
void InitGame( void ) {
  int i;
     
  printf("Starting InitGame() \n");
	
  // load the font pattern
  FntLoad(960,256);
  printf("Fonts loaded: \n");

  for (i = 0; i < 8; i++) 
    fnt_id[i] = FntOpen(0, 16 + i * 16, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 80);
	  
  // save current video mode
  prev_mode = GetVideoMode();

  // init graphic mode
  SetVideoMode( MODE_NTSC);
  printf("Set video mode complete: \n");

  // init the controller buffers
  GetPadBuf(&bb0,&bb1); 
  printf("Set controller buffers complete: \n");

  // all reset, the drawing environment and display are Initialized
  ResetGraph(0);
  GsInitGraph( SCREEN_WIDTH, SCREEN_HEIGHT, GsOFSGPU|GsNONINTER, 0, 0 );
  printf("Screen size setup complete: \n");

  // double buffer definition
  GsDefDispBuff( 0, 0, 0, SCREEN_HEIGHT );
  printf("Double buffer setup complete: \n");  
	 
  // set display output on TV
  GsDISPENV.screen.x = 0;
  GsDISPENV.screen.y = 0;
  GsDISPENV.screen.w = 320;
  GsDISPENV.screen.h = 240;         
  	
  // set up the ordering table handlers
  for( i=0; i < 2; i++ ) {
    world_ordering_table[i].length = 1;
    world_ordering_table[i].org = ordering_table[i];
  }

  // Initializes the ordering table
  GsClearOt( 0, 0, &world_ordering_table[output_buffer_index]);
  GsClearOt( 0, 0, &world_ordering_table[output_buffer_index+1]);
  printf("WOT is setup and complete: \n");
  printf("Game setup is complete: \n");
}// end InitGame

//--------------------------------------------------------------------------
// Function: DeInitGame()
// Description: De-init the game, sound, graphics, etc
// Parameters: none
// Returns: void
// Notes: N/A
//--------------------------------------------------------------------------
void DeInitGame( void ) {
  // set previous video mode
  SetVideoMode( prev_mode );

  // current drawing is cancelled and the command queue is flushed
  ResetGraph(3);    

  printf("Graphics flushed: \n");
  printf("Game now de-init: \n");
}// end DeInitGame

//------------------------------------------------------------------------------
// Function: UpdateScreen()
// Description: Updates all the game objects and redraws the screen
// Parameters: none
// Returns: void
// Notes: 
//------------------------------------------------------------------------------
void UpdateScreen( void ) {
  int i;

  // get the active buffer
  output_buffer_index = GsGetActiveBuff();

  // sets drawing command storage address
  GsSetWorkBase((PACKET*)gpu_work_area[output_buffer_index]);

  // Initializes the ordering table
  GsClearOt(0, 0, &world_ordering_table[output_buffer_index]);

  // rendering done here
  for (i = 0; i < 8; i++) 
    FntFlush(fnt_id[i]);         
		
  // wait for all drawing to be completed
  DrawSync(0);

  // wait for vertical synchronisation
  VSync(0);    // 0: blocking until vertical synch occurs

  // swap double buffers, (changes the display buffer and drawing buffer)
  GsSwapDispBuff();

  // registers drawing clear command in OT (e.g. clear to black)
  GsSortClear(0x0, 0x0, 0x0,
              &world_ordering_table[output_buffer_index]);
   
  // start execution of the drawing command registered in OT
  GsDrawOt(&world_ordering_table[output_buffer_index]);
}// end UpdateScreen 

//----------------------------------EOF-------------------------------------
